package com.care.bms.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.care.bms.mqtt.MqttDataConnector;
import com.care.bms.mqtt.MqttMessageListener;
import com.care.bms.mqtt.MqttDataConnectorPool;
import com.care.common.entity.*;
import com.care.common.enums.*;
import com.care.common.service.*;
import com.care.common.util.CommonConfUtil;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Author: stw
 * @Date: 2021/8/14
 * @Desc:
 */
@Service
public class MqttInitService {
    @Resource
    private CareDeviceService careDeviceService;
    @Resource
    private  MqttMsgService mqttMsgService;

    public void init() {
        QueryWrapper<CareDevice> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().eq(CareDevice::getActiveStatus, DeviceActiveStatusEnum.ACTIVE.getValue());
        List<CareDevice> listCareDevice = careDeviceService.list(queryWrapper);
        for (CareDevice careDevice : listCareDevice) {
            addOne(careDevice);
        }
    }

    /**
     * 添加一个设备到服务
     */
    public void addOne(CareDevice careDevice) {
        if (org.springframework.util.StringUtils.isEmpty(careDevice.getTopic()) || org.springframework.util.StringUtils.isEmpty(careDevice.getClientId())) {
            return;
        }
        MqttDataConnector xbMqttDataConnector = new MqttDataConnector();
        Map<String, String> configParams = new HashMap<>();
        String host = "ws://" + CommonConfUtil.getConf("mqtt.broker") + ":" + CommonConfUtil.getConf("mqtt.port");
        String cliendId = careDevice.getClientId();
        String username = CommonConfUtil.getConf("mqtt.username");
        String password = CommonConfUtil.getConf("mqtt.password");
        String topic = careDevice.getTopic();
        configParams.put("MQTT_HOST", host);
        configParams.put("MQTT_CLIENTID", cliendId);
        configParams.put("MQTT_USERNAME", username);
        configParams.put("MQTT_PASSWORD", password);
        try {
            xbMqttDataConnector.createIotDataSource(configParams);
            MqttMessageListener mqttMessageListener = new MqttMessageListener();
            mqttMessageListener.setMqttMsgService(mqttMsgService);
            Map<String,Thread> threadMap = new HashMap<>();
            mqttMessageListener.setThreadMap(threadMap);
            xbMqttDataConnector.setReSubscribe(topic, mqttMessageListener);

            MqttDataConnectorPool.getMqttDataConnectorMap().put(cliendId, xbMqttDataConnector);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 从服务中去除一个设备
     */
    public void removeOne(CareDevice careDevice) {
        String clientId = careDevice.getClientId();
        if (StringUtils.isEmpty(clientId)) {
            return;
        }
        if (MqttDataConnectorPool.getMqttDataConnectorMap().get(clientId) != null) {
            MqttDataConnectorPool.getMqttDataConnectorMap().get(clientId).destroy();
            MqttDataConnectorPool.getMqttDataConnectorMap().remove(clientId);
        }
    }

}
